ChildRef

Enum ChildRef 

pub enum ChildRef<'a, C: PageTableConfig> {
    PageTable(PageTableNodeRef<'a, C>),
    Frame(Paddr, PagingLevel, PageProperty),
    None,
}
A reference to the child of a page table node.

Verification Design

If the child is itself a page table node, it is represented by a PageTableNodeRef, because a reference to it must be treated as a potentially shared reference of the appropriate lifetime. By contrast, a mapped frame can be referenced by just carrying its values, and an absent one is just a simple tag.

Variants



PageTable(PageTableNodeRef<'a, C>)

A child page table node.



Frame(Paddr, PagingLevel, PageProperty)

Physical address of a mapped physical frame.

It is associated with the virtual page property and the level of the mapping node, which decides the size of the frame.



None

Implementations

Source

impl<'a, C: PageTableConfig> ChildRef<'a, C>

pub fn arrow_1(self) -> PagingLevel

pub fn arrow_2(self) -> PageProperty

pub fn arrow_PageTable_0(self) -> PageTableNodeRef<'a, C>

pub fn arrow_Frame_0(self) -> Paddr

pub fn arrow_Frame_1(self) -> PagingLevel

pub fn arrow_Frame_2(self) -> PageProperty

Source

impl<C: PageTableConfig> ChildRef<'_, C>

pub exec fn from_pte(pte: &C::E, level: PagingLevel) -> res : Self

with
Tracked(regions): Tracked<&mut MetaRegionOwners>,
Tracked(entry_owner): Tracked<&EntryOwner<C>>,
requires
entry_owner.pte_invariants(*pte, *old(regions)),
level == entry_owner.parent_level,
ensures
res.invariants(*entry_owner, *regions),
*regions =~= *old(regions),

Converts a PTE to a reference to a child.

Verified Properties
Preconditions
  • Safety Invariants: the PTE’s safety invariants must hold.
  • Safety: level must match the original level of the child.
Postconditions
  • Safety Invariants: the safety invariants are preserved.
  • Correctness: the ChildRef is equivalent to the original PTE.
  • Safety: No frame other than the target entry’s (if applicable) is impacted by the call.
Safety
  • The PTE safety invariants require that the PTE was previously obtained using Self::from_pte
  • The soundness of using the resulting ChildRef as a reference follows from FrameRef safety.
Source

impl<C: PageTableConfig> ChildRef<'_, C>

pub open spec fn invariants(self, owner: EntryOwner<C>, regions: MetaRegionOwners) -> bool

{
    &&& owner.inv()
    &&& regions.inv()
    &&& self.wf(owner)
    &&& owner.relate_region(regions)
    &&& !owner.in_scope

}