pub exec fn nr_pte_index_bits<C: PagingConstsTrait>() -> usizeExpand description
returns
nr_pte_index_bits_spec::<C>(),The number of virtual address bits used to index a PTE in a page.
pub exec fn nr_pte_index_bits<C: PagingConstsTrait>() -> usizenr_pte_index_bits_spec::<C>(),The number of virtual address bits used to index a PTE in a page.